#!/usr/bin/python

# Copyright (c) 2015 Justin Hughes (jhughes@cavs.msstate.edu)

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

#The Savitzky-Golay filter is a least-squares polynomial smoothing routine.
#  It uses N (window_size) datapoints to locally fit a polynomial

import sys
import os
import numpy as np
import matplotlib.pyplot as plt
from math import factorial


# Reproduced from SciPy cookbook example.
def sg_filter(y, window_size, order, deriv=0, rate=1):
    try:
        window_size = np.abs(np.int(window_size))
        order = np.abs(np.int(order))
    except ValueError, msg:
        raise ValueError("window_size and order have to be of type int")
    if window_size%2 != 1 or window_size < 1:
        raise TypeError("window_size must be a positive odd number")
    if window_size < order + 2:
        raise TypeError("window_size is too small for the polynomial order")
    order_range = range(order+1)
    half_window = (window_size - 1) // 2

    #compute coefficients
    b = np.mat([[k**i for i in order_range] for k in range(-half_window,half_window+1)])
    m = np.linalg.pinv(b).A[deriv]*rate**deriv*factorial(deriv)
    #pad the signal at the extremes with values taken from the signal
    firstvals = y[0] - np.abs(y[1:half_window+1][::-1] - y[0])
    lastvals = y[-1] + np.abs(y[-half_window-1:-1][::-1] - y[-1])
    y = np.concatenate((firstvals,y,lastvals))
    return np.convolve(m[::-1], y, mode = 'valid')


if __name__ == '__main__':
    # Read data file using NumPy (automatically converts to float, ignores header)
    filename = str(raw_input("Input file: "))

    data = np.genfromtxt(filename+'.csv',delimiter=',',skip_header=1)
    np.set_printoptions(suppress=True)

    numcols = len(data[0])
    numsets = numcols/2


    # Terminates script if uneven number of columns
    if(numcols%2 != 0):
        sys.exit()

    x = data[:,0]
    y = data[:,1]

    #Number of datapoints used to locally fit specified order polynomial
    window_size = abs(int(raw_input("Window size: ")))

    if window_size%2 == 0:
        window_size +=1

    order = int(raw_input("Polynomial Order: "))
        
    if(os.path.isfile(filename+'_SMOOTHED.csv')):
        os.remove(filename+'_SMOOTHED.csv')

    #x_new = sg_filter(x,window_size,order,deriv=0,rate=1)
    y_new = sg_filter(y,window_size,order,deriv=0,rate=1)
    
    for i in range(0,len(y)-1):
        if y[i] == 0 or str(y[i]) == 'nan':
            y[i] = 0.00001
            
    fig, ax1 = plt.subplots()
    plt.title("Order=%d, Numpoints=%d" %(order, window_size))
    ax1.plot(x,y,'k-',linewidth=2,label="ORIGINAL")
    ax1.plot(x,y_new,'r-',linewidth=2,label="SMOOTHED, ORDER=%d" %order)
    ax1.set_xlabel("X",fontsize=20)
    ax1.set_ylabel("Y",fontsize=20,color='k')
    
    x = x.reshape(len(x),1)
    y_new = y_new.reshape(len(y_new),1)

    

    dataout = np.hstack((x,y_new))
    np.savetxt(filename+'_SMOOTHED.csv',dataout,fmt='%10.6f',delimiter=",",header='X, Y_Smoothed')
    #plt.legend(loc='best')
    plt.savefig(filename)
    sys.exit()
